/* This is "bisect2.c", written to accompany Introduction to
   Scientific Programming by Joseph L. Zachary. */

/* This program is an implementation and a test driver for the
   bisection method as described in Chapter 9 of the text. */


#include <stdio.h>
#include <math.h>



/* This is x^4 + 2x^3 - 14x^2 + 2x + 1, whose root we seek. */

double f (double x)
{
  return(1 + x*(2 + x*(-14 + x*(2 + x))));
}



/* Uses the bisection method to find an approximate root for "f",
   where f(pos) > 0 and f(neg) < 0.  It narrows down the difference
   between "pos" and "neg" until f applied to their average is
   less than 10^-10. */

double bisection (double pos, double neg)
{

  double avg;   /* Keeps track of the average of "pos" and "neg". */
  

  /* Apply the bisection narrowing process until f(avg) is small.
     At that point, report "avg" as an approximate root of "f". */

  avg = (pos + neg) / 2.0;
  
  while (fabs(f(avg)) > 1e-10) {
    avg = (pos + neg) / 2.0;
    if (f(avg) >= 0) {
      pos = avg;
    }
    else {
      neg = avg;
    }
  }
  
  return(avg);
}



/* Prompts for the positive and negative approximations to the root of
   the function "f" defined above, then computes and displays an
   approximation to f's root. */

void main ()
{

  double positive,    /* "Positive" guess, meaning f(positive) > 0 */
         negative,    /* "Negative" guess, meaning f(negative) < 0 */
         root;        /* Approximate root of "f". */


  /* Obtain a valid positive guess. */

  do {
    printf("Enter positive guess: ");
    scanf("%lf", &positive);
  }
  while (f(positive) <= 0);


  /* Obtain a valid negative guess. */

  do {
    printf("Enter negative guess: ");
    scanf("%lf", &negative);
  }
  while (f(negative) >= 0);


  /* Obtain and display an approximate root of "f". */

  root = bisection(positive, negative);
  printf("An approximate root is %.10g\n", root);
  printf("f(approximate root) is %.10g\n", f(root));

}
